package com.xypower.mppreview;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.ImageDecoder;
import android.graphics.ImageFormat;
import android.hardware.camera2.CameraCharacteristics;
import android.hardware.camera2.CaptureResult;
import android.hardware.camera2.DngCreator;
import android.media.Image;
import android.media.ImageReader;
import android.util.Log;

import androidx.annotation.NonNull;

import com.xypower.mppreview.bean.PngPhotoBean;
import com.xypower.mppreview.interfaces.CompleteCallback;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;

public class ImageSaver implements Runnable {

    private final static String TAG = "HDR";
    private final Image mImage;
    private final File mFile;
    private final CaptureResult mCaptureResult;
    private final CameraCharacteristics mCharacteristics;
    private CompleteCallback mCallback;
    private final ImagePair mImagePair;

    private final Camera2RawFragment.RefCountedAutoCloseable<ImageReader> mReader;

    public static class ImageInfo {
        public long exposureTime;
        public int length;
        public Bitmap bitmap;

        ImageInfo(Bitmap bmp, int length, long exp) {
            this.exposureTime = exp;
            this.length = length;
            bitmap = bmp;
        }
    }

    public static class ImagePair {
        public List<ImageInfo> mImages;
        public int mExpectedCount;
        public Runnable mRunnable;

        public ImagePair(int expectedCount) {
            mImages = new ArrayList<>();
            mExpectedCount = expectedCount;
            mRunnable = null;
        }

        public void setRunnable(Runnable runnable) {
            mRunnable = runnable;
        }

        public void addImage(Bitmap bitmap, int length, long exp) {
            boolean isFull = false;
            ImageInfo imageInfo = new ImageInfo(bitmap, length, exp);
            synchronized (mImages) {
                mImages.add(imageInfo);
                isFull = (mImages.size() == mExpectedCount);
            }

            if (mRunnable != null && isFull) {
                mRunnable.run();
            }
        }

        public List<ImageInfo> getImages() {
            return mImages;
        }
    }

    public static abstract class ImagePairRunnable implements Runnable {

        protected ImagePair mImagePair;

        public ImagePairRunnable(ImagePair imagePair) {
            mImagePair = imagePair;
        }

    }

    private ArrayList<PngPhotoBean> mlist = new ArrayList<>();//用来存储已拍照的照片名称

    public ImageSaver(Image image, File file, CaptureResult result, CameraCharacteristics characteristics,
                      Camera2RawFragment.RefCountedAutoCloseable<ImageReader> reader, ArrayList<PngPhotoBean> list,
                      CompleteCallback callback, ImagePair imagePair) {
        mImage = image;
        mFile = file;
        mCaptureResult = result;
        mCharacteristics = characteristics;
        mReader = reader;
        mlist = list;
        mCallback = callback;
        mImagePair = imagePair;
    }

    @Override
    public void run() {
        boolean success = false;
        int format = mImage.getFormat();
        switch (format) {
            case ImageFormat.JPEG: {
                ByteBuffer buffer = mImage.getPlanes()[0].getBuffer();
                byte[] bytes = new byte[buffer.remaining()];
                buffer.get(bytes);
                FileOutputStream output = null;
                try {
                    output = new FileOutputStream(mFile);
                    output.write(bytes);
                    success = true;
                } catch (IOException e) {
                    e.printStackTrace();
                } finally {
                    mImage.close();
                    closeOutput(output);
                }
                break;
            }
            case ImageFormat.RAW_SENSOR: {
                DngCreator dngCreator = new DngCreator(mCharacteristics, mCaptureResult);

                if (mImagePair != null) {
                    ByteBuffer byteBuffer = null;
                    ByteBufferOutputStream baos = null;

                    Long t = mCaptureResult.get(CaptureResult.SENSOR_EXPOSURE_TIME);

                    try {
                        byteBuffer = ByteBuffer.allocateDirect(mImage.getWidth() * mImage.getHeight() * 2 + 81768);
                        baos = new ByteBufferOutputStream(byteBuffer);
                        Log.d(TAG, "Before Saving DNG Exp=" + t.toString());
                        dngCreator.writeImage(baos, mImage);
                        byteBuffer.limit(byteBuffer.position());
                        byteBuffer.flip();
                        Log.d(TAG, "After Saving DNG Exp=" + t.toString() + " size=" + byteBuffer.limit());
                        ImageDecoder.OnHeaderDecodedListener listener = new ImageDecoder.OnHeaderDecodedListener() {
                            @Override
                            public void onHeaderDecoded(@NonNull ImageDecoder decoder, @NonNull ImageDecoder.ImageInfo info, @NonNull ImageDecoder.Source source) {
                                // decoder.setAllocator(ImageDecoder.ALLOCATOR_SOFTWARE);
                                decoder.setAllocator(ImageDecoder.ALLOCATOR_HARDWARE);
                                // decoder.setTargetSize(4702, 3520);
                            }
                        };

                        Log.i(TAG, "Start Hardware Decoding Exp=" + t.toString() + " TID=" + Thread.currentThread().getId());
                        ImageDecoder imageDecoder = null;
                        Bitmap bmp = null;
                        // ImageDecoder.Source source = ImageDecoder.createSource(mFile);
                        ImageDecoder.Source source = ImageDecoder.createSource(byteBuffer);


                        try {
                            bmp = ImageDecoder.decodeBitmap(source, listener);
                        } catch (Exception ex) {
                            ex.printStackTrace();
                        }
                        Log.i(TAG, "End Hardware Decoding Exp=" + t.toString());


                        byteBuffer.clear();
                        byteBuffer = null;
                        mImagePair.addImage(bmp, 0, t.longValue());
//                        bmp.recycle();
//                        bmp = null;
                        success = true;
                    } catch (Exception e) {
                        e.printStackTrace();
                    } finally {
                        dngCreator.close();
                        mImage.close();
                        closeOutput(baos);
                    }
                } else {
                    FileOutputStream output = null;
                    try {
                        output = new FileOutputStream(mFile);
                        dngCreator.writeImage(output, mImage);
                        success = true;
                    } catch (IOException e) {
                        e.printStackTrace();
                    } finally {
                        mImage.close();
                        dngCreator.close();
                        closeOutput(output);
                    }
                }

                break;
            }
            default: {
                break;
            }
        }
        mReader.close();

    }

    private static void closeOutput(OutputStream outputStream) {
        if (null != outputStream) {
            try {
                outputStream.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

//    private static void clear() {
//        if (null != outputStream) {
//            try {
//                outputStream.close();
//            } catch (IOException e) {
//                e.printStackTrace();
//            }
//        }
//    }

//    private void saveJpeg(Image image,String name) {
//        Image.Plane[] planes = image.getPlanes();
//        ByteBuffer buffer = planes[0].getBuffer();
//        int pixelStride = planes[0].getPixelStride();
//        int rowStride = planes[0].getRowStride();
//        int rowPadding = rowStride - pixelStride * mWidth;
//
//        Bitmap bitmap = Bitmap.createBitmap(mWidth + rowPadding / pixelStride, mHeight, Bitmap.Config.ARGB_8888);
//        bitmap.copyPixelsFromBuffer(buffer);
//        //bitmap.compress(Bitmap.CompressFormat.JPEG, 100, fos);
//        ImageSaveUtil.saveBitmap2file(bitmap,getApplicationContext(),name);
//
//    }
}